@@ -1150,10 +1150,20 @@ int XrdOssSys::xdefault(XrdOucStream &Config, XrdSysError &Eroute)
 
 int XrdOssSys::xfdlimit(XrdOucStream &Config, XrdSysError &Eroute)
 {
+    char *val;
+    int fence = 0, FDHalf = FDLimit>>1;
+
+    if (!(val = Config.GetWord()))
+       {Eroute.Emsg("Config", "fdlimit fence not specified"); return 1;}
+
+    if (!strcmp(val, "*")) FDFence = FDHalf;
+       else {if (XrdOuca2x::a2i(Eroute,"fdlimit fence",val,&fence,0)) return 1;
+             FDFence = (fence < FDHalf ? fence : FDHalf);
+            }
 
     while(Config.GetWord()) {}
 
-    Eroute.Say("Config warning: ", "fdlimit directive no longer supported.");
+//  Eroute.Say("Config warning: ", "fdlimit directive no longer supported.");
 
     return 0;
 }
@@ -1326,9 +1336,19 @@ int XrdOssSys::xpath(XrdOucStream &Config, XrdSysError &Eroute)
    pP = XrdOucExport::ParsePath(Config, Eroute, RPList, DirFlags);
    if (!pP) return 1;
 
-// This plugin does not support relative paths so check for this
+// If this is an absolute path, we are done
 //
    if (*(pP->Path()) == '/') return 0;
+
+// If this is an objectid path then make sure to set the default for these
+//
+   if (*(pP->Path()) == '*')
+      {RPList.Defstar(pP->Flag());
+       return 0;
+      }
+
+// We do not (yet) support exporting specific object ID's
+//
    Eroute.Emsg("Config", "Unsupported export -", pP->Path());
    return 1;
 }
